#include <iostream>
#include <iostream>

/**
 * @brief Una struttura per gli array di interi
 */
struct IntArray {
private:
  size_t len;   //! è la lughezze dell'array
  int* values;  //! è il puntatore all'area di memoria degli elementi

public:
  /**
   * @brief Un costruttore
   * 
   * @param length è la lunghezza del nuovo array
   */
  IntArray(const size_t length)
    : len{length}, values{new int[length]}   // inizializzo i membri
  {
    std::cout << "IntArray(" << length << ")" << std::endl;
  }

  /**
   * @brief Un costruttore
   */
  IntArray()
    : IntArray(0)   // in fase di inizializzazione posso invocare 
                    // un altro costruttore
  {
    std::cout << "IntArray()" << std::endl;
  }

  /**
   * @brief Un costruttore
   * 
   * @param length è la lunghezza del nuovo array
   * @param init_value è il valore a cui vengono inizializzati gli elementi
   */
  IntArray(const size_t length, const int& init_value)
    : IntArray(length)
  {
    std::cout << "IntArray(" << length << "," << init_value << ")" << std::endl;

    for (size_t i=0; i<len; ++i) {
      values[i] = init_value;
    }
  }

  /**
   * @brief Accede all'i-esimo elemento dell'array
   * 
   * @param i è l'indice dell'elemento ambito
   * @return un riferiemento all'elemento dell'array
   */
  int& at(size_t i) {           // questo metodo non è const perché consente
                                // di modificare l'istanza dell'oggetto
    return values[i];
  }

  /**
   * @brief Accede all'i-esimo elemento dell'array
   * 
   * @param i è l'indice dell'elemento ambito
   * @return un riferiemento all'elemento dell'array
   */
  const int& at(size_t i) const {  // questo metodo è const perché non consente
                                   // di modificare l'istanza dell'oggetto
    return values[i];
  }

  /**
   * @brief Ottieni la lunghezza dell'array
   * 
   * @return la lunghezza dell'array
   */
  const size_t& size() const {
    return len; 
  }

  /**
   * @brief Stampa l'array
   */
  void println() const {
    std::cout << "[";
    for (size_t i=0; i<len; ++i) {
      if (i>0) {
        std::cout <<  ",";
      }

      std::cout << values[i];
    }
    std::cout << "]" << std::endl;
  }

  /**
   * @brief Il distruttore
   */
  ~IntArray()         // viene invocato ogni volta che l'oggetto viene distrutto
  {
    std::cout << "~IntArray()" << std::endl;
    delete[] values;
  }
};

int main() {
  std::cout << "Costruisco a1" << std::endl;
  IntArray a1;
  std::cout << "Costruisco a2" << std::endl;
  IntArray a2(3);
  std::cout << "Costruisco a3" << std::endl;
  IntArray a3(4, -7);

  std::cout << "Costruisco p" << std::endl;
  IntArray* p = new IntArray(5, 10);

  a1.println();
  a2.println();
  a3.println();
  p->println();

  a3.at(2) = 5 + a3.at(0);
  p->at(2) = 2 + p->at(0);

  a1.println();
  a2.println();
  a3.println();
  p->println();

  std::cout << "Cancello p" << std::endl;
  delete p;

  std::cout << "Fine" << std::endl;
  return 0; 
}
